@@ -5,10 +5,11 @@
 #define __EXCHANGE_ACCOUNT_H__
 
 #include "exchange-types.h"
-#include "e2k-connection.h"
+#include "e2k-autoconfig.h"
+#include "e2k-context.h"
 #include "e2k-global-catalog.h"
 #include "e2k-security-descriptor.h"
-#include <shell/e-folder.h>
+#include "e-folder.h"
 #include <e-util/e-account-list.h>
 
 #ifdef __cplusplus
@@ -33,26 +34,25 @@ struct _ExchangeAccount {
 	 */
 	char *account_name, *account_filename, *storage_dir;
 	char *exchange_server, *home_uri, *public_uri;
-	char *legacy_exchange_dn;
+	char *legacy_exchange_dn, *default_timezone;
 
-	gboolean filter_inbox;
+	gboolean filter_inbox, filter_junk, filter_junk_inbox_only;
 };
 
 struct _ExchangeAccountClass {
 	GObjectClass parent_class;
 
 	/* signals */
-	void (*connected) (ExchangeAccount *);
+	void (*connected) (ExchangeAccount *, E2kContext *);
 
 	void (*new_folder) (ExchangeAccount *, EFolder *);
 	void (*removed_folder) (ExchangeAccount *, EFolder *);
-	void (*updated_folder) (ExchangeAccount *, EFolder *);
 };
 
 GType                  exchange_account_get_type             (void);
 ExchangeAccount       *exchange_account_new                  (EAccountList                   *account_list,
 							      EAccount                       *adata);
-E2kConnection         *exchange_account_get_connection       (ExchangeAccount                *acct);
+E2kContext            *exchange_account_get_context          (ExchangeAccount                *acct);
 E2kGlobalCatalog      *exchange_account_get_global_catalog   (ExchangeAccount                *acct);
 
 const char            *exchange_account_get_standard_uri     (ExchangeAccount                *acct,
@@ -65,19 +65,22 @@ char                  *exchange_account_get_foreign_uri      (ExchangeAccount
 							      E2kGlobalCatalogEntry          *entry,
 							      const char                     *std_uri_prop);
 
-typedef void         (*ExchangeAccountConnectCallback)       (ExchangeAccount                *acct,
-							      gpointer                        user_data);
-void                   exchange_account_async_connect        (ExchangeAccount                *acct,
-							      ExchangeAccountConnectCallback  callback,
-							      gpointer                        user_data);
+E2kContext            *exchange_account_connect              (ExchangeAccount                *acct);
+
+EFolder               *exchange_account_get_folder           (ExchangeAccount                *acct,
+							      const char                     *path_or_uri);
+GPtrArray             *exchange_account_get_folders          (ExchangeAccount                *acct);
+
+void                   exchange_account_rescan_tree          (ExchangeAccount                *acct);
+
+char 		       *exchange_account_get_password (ExchangeAccount *acct);
+
+void		       exchange_account_set_password (ExchangeAccount *acct,
+							char *old_password,
+							char *new_password);
+void 		      exchange_account_forget_password (ExchangeAccount *acct);
 
-EFolder               *exchange_account_get_folder             (ExchangeAccount                *acct,
-								const char                     *path_or_uri);
-GPtrArray             *exchange_account_get_folders            (ExchangeAccount                *acct);
-void                   exchange_account_update_folder          (ExchangeAccount                *acct,
-								EFolder                        *folder);
 
-void                   exchange_account_rescan_tree            (ExchangeAccount                *acct);
 
 typedef enum {
 	EXCHANGE_ACCOUNT_FOLDER_OK,
@@ -89,43 +92,33 @@ typedef enum {
 	EXCHANGE_ACCOUNT_FOLDER_UNSUPPORTED_OPERATION,
 	EXCHANGE_ACCOUNT_FOLDER_GENERIC_ERROR
 } ExchangeAccountFolderResult;
-typedef void (*ExchangeAccountFolderCallback) (ExchangeAccount *account,
-					       ExchangeAccountFolderResult,
-					       EFolder *folder,
-					       gpointer user_data);
 
-void exchange_account_async_create_folder (ExchangeAccount *account,
-					   const char      *path,
-					   const char      *type,
-					   ExchangeAccountFolderCallback,
-					   gpointer         user_data);
-void exchange_account_async_remove_folder (ExchangeAccount *account,
-					   const char      *path,
-					   ExchangeAccountFolderCallback,
-					   gpointer         user_data);
-void exchange_account_async_xfer_folder   (ExchangeAccount *account,
-					   const char      *source_path,
-					   const char      *dest_path,
-					   gboolean         remove_source,
-					   ExchangeAccountFolderCallback,
-					   gpointer         user_data);
-void exchange_account_async_open_folder   (ExchangeAccount *account,
-					   const char      *path,
-					   ExchangeAccountFolderCallback,
-					   gpointer         user_data);
+ExchangeAccountFolderResult exchange_account_create_folder (ExchangeAccount *account,
+							    const char      *path,
+							    const char      *type);
+ExchangeAccountFolderResult exchange_account_remove_folder (ExchangeAccount *account,
+							    const char      *path);
+ExchangeAccountFolderResult exchange_account_xfer_folder   (ExchangeAccount *account,
+							    const char      *source_path,
+							    const char      *dest_path,
+							    gboolean         remove_source);
+ExchangeAccountFolderResult exchange_account_open_folder   (ExchangeAccount *account,
+							    const char      *path);
 
-void exchange_account_async_discover_shared_folder  (ExchangeAccount *account,
-						     const char      *user,
-						     const char      *folder,
-						     ExchangeAccountFolderCallback,
-						     gpointer         user_data);
-void exchange_account_cancel_discover_shared_folder (ExchangeAccount *account,
-						     const char      *user,
-						     const char      *folder);
-void exchange_account_async_remove_shared_folder    (ExchangeAccount *account,
-						     const char      *path,
-						     ExchangeAccountFolderCallback,
-						     gpointer         user_data);
+ExchangeAccountFolderResult exchange_account_discover_shared_folder  (ExchangeAccount *account,
+								      const char      *user,
+								      const char      *folder_name,
+								      EFolder        **folder);
+void                  exchange_account_cancel_discover_shared_folder (ExchangeAccount *account,
+								      const char      *user,
+								      const char      *folder);
+ExchangeAccountFolderResult exchange_account_remove_shared_folder    (ExchangeAccount *account,
+								      const char      *path);
+
+ExchangeAccountFolderResult exchange_account_add_favorite            (ExchangeAccount *account,
+								      EFolder         *folder);
+ExchangeAccountFolderResult exchange_account_remove_favorite         (ExchangeAccount *account,
+								      EFolder         *folder);
 
 #ifdef __cplusplus
 }
